
# q-expansion of newform 7056.2.a.t, downloaded from the LMFDB on 18 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7056
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-1], [0], [3], [-4], [0], [-4], [8], [3], [-5], [8], [8], [-6], [-10], [-9], [5], [10], [-6], [10], [-2], [-11], [-7], [-18], [17], [-2], [0], [-11], [-10], [8], [7], [-15], [14], [-22], [-18], [5], [4], [-8], [2], [2], [12], [0], [-16], [-27], [26], [12], [-6], [1], [-21], [12], [24], [0], [-23], [-11], [14], [-14], [1], [-17], [8], [-6], [-18], [-19], [-16], [0], [-9], [-13], [-4], [-15], [-20], [-2], [8], [6], [-7], [16], [16], [-2], [-26], [-28], [8], [-7], [0], [-6], [-12], [2], [-9], [-1], [-4], [-17], [30], [16], [28], [-6], [-37], [-9], [-10], [12], [-39], [-18], [-12], [34], [8], [-15], [-39], [-12], [-22], [33], [-45], [0], [-42], [13], [13], [8], [30], [6], [11], [22], [46], [50], [-15], [24], [-38], [29], [19], [17], [0], [-5], [6], [34], [47], [-14], [-10], [34], [27], [-10], [-12], [-37], [-26], [-22], [3], [16], [2], [39], [-32], [-41], [4], [24], [-14], [14], [-14], [22], [-32], [-54], [-40], [48], [16], [6], [16], [42], [-19], [-13], [-48], [38], [-3], [-25], [22], [-44], [-15], [26], [15], [-21], [-19], [18], [4], [-27], [-11], [-4], [46], [45], [-24], [6], [37], [51], [-26], [14], [26], [31], [-26], [42], [59], [20], [31], [21], [-52], [58], [7], [18], [-26], [-66], [62], [22], [1], [64], [42], [38], [-16], [69], [24], [21], [-56], [-34], [-62], [11], [-35], [-61], [-28], [-18], [-1], [56], [42], [2], [8], [-64], [-58], [-31], [-33], [64], [26], [28], [-9], [12], [-6], [-8], [-51], [-30], [-2], [60], [13], [21], [-68], [16], [60], [16], [17], [50], [36], [-12], [19], [36], [16], [-64], [26], [-78], [6], [-18], [49], [-21], [-30], [-28], [1], [67], [-40], [15], [-54], [76], [2], [-70], [5], [60], [-20], [29], [48], [10], [59], [51], [43], [-56], [29], [-40], [54], [32], [22], [17], [-36], [36], [50], [-8], [79], [45], [53], [84], [-10], [-53], [-12], [-20], [4], [3], [56], [-86], [60], [-62], [-6], [-11], [59], [39], [-10], [-14], [-9], [-38], [6], [-26], [-30], [-78], [-20], [0], [-32], [-38], [-51], [-48], [51], [8], [64], [34], [-19], [56], [24], [5], [50], [-62], [-38], [-15], [-54], [-51], [-72], [40], [82], [44], [52], [-2], [-81], [-47], [14], [-36], [-75], [-81], [-13], [9], [10], [14], [30], [45], [-68], [27], [42], [93], [-50], [-6], [-42], [-43], [-12], [-34], [90], [-24], [-54], [-20], [-32], [-53], [22], [-16], [57], [-51], [80], [46], [43], [8], [-88], [-56], [20], [21], [-22], [-46], [-6], [-6], [8], [-36], [-78], [14], [8], [-28], [28], [12], [-71], [42], [-33], [46], [-58], [-89], [5], [-30], [40], [21], [-4], [-24], [76], [-54], [65], [25], [54], [38], [-14], [-63], [41], [33], [-23], [-82], [-86], [-63], [-12], [32], [72], [-18], [49], [-54], [58], [17], [-10], [78], [69], [-48], [30], [-52]]
